from django.forms import ModelForm
from django.test import TestCase

from andablog import models


class TestEntryEditing(TestCase):

    def setUp(self):
        self.entry = models.Entry.objects.create(title=u'Welcome!', content='### Some Content', is_published=False)

        class EntryForm(ModelForm):
            class Meta:
                model = models.Entry
                fields = ['title', 'content', 'is_published']

        self.form_cls = EntryForm

    def test_form_editing(self):
        """Should be able to properly edit an entry within a model form"""
        update = {
            'title': 'Last Post (Final)',
            'content': '### Goodbye!',
            'is_published': True,
        }

        form = self.form_cls(update, instance=self.entry)

        form.save()

        actual = models.Entry.objects.get(pk=self.entry.pk)
        self.assertEquals(actual.title, update['title'])
        self.assertEquals(actual.content.raw, update['content'])
        self.assertIsNotNone(actual.published_timestamp)


class TestEntryCreation(TestCase):

    def setUp(self):
        class EntryForm(ModelForm):
            class Meta:
                model = models.Entry
                fields = ['title', 'content', 'is_published']

        self.form_cls = EntryForm

    def test_form_create(self):
        """Should be able to properly create an entry within a model form"""
        create = {
            'title': 'Last Post (Final)',
            'content': '### Goodbye!',
            'is_published': False,
        }

        form = self.form_cls(create)
        print(form.errors)

        form.save()

        actual = models.Entry.objects.get(slug='last-post-final')
        self.assertEquals(actual.title, create['title'])
        self.assertEquals(actual.content.raw, create['content'])
        self.assertIsNone(actual.published_timestamp)



import six
from django.db import models

from authtools.models import AbstractEmailUser
from django.template.defaultfilters import slugify


class User(AbstractEmailUser):
    name = models.CharField(max_length=100)
    profile_name = models.CharField('profile name', max_length=20, unique=True)
    slug = models.SlugField(max_length=50, unique=True)

    REQUIRED_FIELDS = ['name', 'profile_name']  # Base already includes email

    def save(self, *args, **kwargs):
        self.slug = slugify(self.profile_name)
        super(User, self).save()

    def get_full_name(self):
        return self.name

    def get_short_name(self):
        """Required by And-a-Blog for displaying author of a (blog/comment)"""
        return self.profile_name

    def get_absolute_url(self):
        """Since it's provided And-a-Blog will use this to link to an author's profile"""
        return self.userprofile.get_absolute_url()

    def __unicode__(self):
        return six.text_type(self.get_short_name())



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django import VERSION as DJANGO_VERSION


def get_operations():
    """
    This will break things if you upgrade Django to 1.8 having already applied this migration in 1.7.
    Since this is for a demo site it doesn't really matter (simply blow away the DB if you want to go to 1.8)

    Our demo site is a unusual in that we want to run it's tests (for integration testing) in multiple Django versions.
    Typical sites don't have to worry about that sort of thing.
    """
    compatible = (1, 8) <= DJANGO_VERSION < (1, 10)
    if not compatible:
        return []

    return [
        migrations.AlterField(
            model_name='user',
            name='groups',
            field=models.ManyToManyField(related_query_name='user', related_name='user_set', to='auth.Group', blank=True, help_text='The groups this user belongs to. A user will get all permissions granted to each of their groups.', verbose_name='groups'),
        ),
        migrations.AlterField(
            model_name='user',
            name='last_login',
            field=models.DateTimeField(null=True, verbose_name='last login', blank=True),
        ),
    ]


class Migration(migrations.Migration):

    dependencies = [
        ('common', '0001_initial'),
    ]

    operations = get_operations()


from django.contrib import admin

from .models import UserProfile


class UserProfileAdmin(admin.ModelAdmin):

    list_display = (
        'user',
        'greatest_fear',
    )
    search_fields = (
        'user__email',
        'user__name',
        'user__profile_name',
        'user__slug',
        'greatest_fear',
        'home',
    )

admin.site.register(UserProfile, UserProfileAdmin)



from django.test import SimpleTestCase
from django.utils.safestring import SafeString

from .. templatetags import andablog_tags


class TestAuthorDisplay(SimpleTestCase):

    def setUp(self):
        class MockAuthor(object):
            def get_short_name(self):
                return 'ShyGuy'
        self.author = MockAuthor()

    def test_none_link(self):
        """Test that our template tag returns just the short name when we get None for the URL"""
        def mock_get_url():
            return None
        self.author.get_absolute_url = mock_get_url

        display = andablog_tags.author_display(self.author)
        self.assertEqual(display, 'ShyGuy')

    def test_empty_link(self):
        """Test that our template tag returns just the short name when we get "" for the URL"""
        def mock_get_url():
            return ""
        self.author.get_absolute_url = mock_get_url

        display = andablog_tags.author_display(self.author)
        self.assertEqual(display, 'ShyGuy')

    def test_no_link_function(self):
        """Test that our template tag returns just the short name when we can not get the URL"""
        display = andablog_tags.author_display(self.author)
        self.assertEqual(display, 'ShyGuy')

    def test_profile_display(self):
        """Test that our template tag returns the short name hyperlinked to the URL"""
        def mock_get_url():
            return 'http://example.com/profile/shyguy'
        self.author.get_absolute_url = mock_get_url

        display = andablog_tags.author_display(self.author)
        self.assertEqual(display, SafeString('<a href="http://example.com/profile/shyguy">ShyGuy</a>'))

    def test_no_short_name(self):
        """Test that our template tag uses the unicode representation when there is no short name"""
        class MockAuthor(object):
            pass
        self.author = MockAuthor()

        display = andablog_tags.author_display(self.author)
        self.assertIn('MockAuthor object', display)



# -*- coding: utf-8 -*-
import time

from django.core.urlresolvers import reverse
from django.db import models
from django.template.defaultfilters import truncatechars
from django.utils.text import slugify
from django.utils import timezone
from django.conf import settings

from markitup.fields import MarkupField
from model_utils.models import TimeStampedModel
from taggit.managers import TaggableManager


class Entry(TimeStampedModel):
    """
    Represents a blog Entry.
    Uses TimeStampModel to provide created and modified fields
    """
    title = models.CharField(max_length=255)
    slug = models.SlugField(max_length=255, unique=True, editable=False)
    content = MarkupField()
    is_published = models.BooleanField(default=False)
    published_timestamp = models.DateTimeField(blank=True, null=True, editable=False)
    author = models.ForeignKey(settings.AUTH_USER_MODEL, null=True, editable=True)
    tags = TaggableManager(blank=True)

    def __unicode__(self):
        return self.title

    class Meta:
        verbose_name_plural = "entries"

    def get_absolute_url(self):
        return reverse('andablog:entrydetail', args=[self.slug])

    def _insert_timestamp(self, slug, max_length=255):
        """Appends a timestamp integer to the given slug, yet ensuring the
        result is less than the specified max_length.
        """
        timestamp = str(int(time.time()))
        ts_len = len(timestamp) + 1
        while len(slug) + ts_len > max_length:
            slug = '-'.join(slug.split('-')[:-1])
        slug = '-'.join([slug, timestamp])
        return slug

    def _slugify_title(self):
        """Slugify the Entry title, but ensure it's less than the maximum
        number of characters. This method also ensures that a slug is unique by
        appending a timestamp to any duplicate slugs.
        """
        # Restrict slugs to their maximum number of chars, but don't split mid-word
        self.slug = slugify(self.title)
        while len(self.slug) > 255:
            self.slug = '-'.join(self.slug.split('-')[:-1])

        # Is the same slug as another entry?
        if Entry.objects.filter(slug=self.slug).exclude(id=self.id).exists():
            # Append time to differentiate.
            self.slug = self._insert_timestamp(self.slug)

    def save(self, *args, **kwargs):
        self._slugify_title()

        # Time to publish?
        if not self.published_timestamp and self.is_published:
            self.published_timestamp = timezone.now()
        elif not self.is_published:
            self.published_timestamp = None

        super(Entry, self).save(*args, **kwargs)


class EntryImage(TimeStampedModel):
    entry = models.ForeignKey(Entry)
    image = models.ImageField(blank=True, upload_to='andablog/images')

    @property
    def image_url(self):
        return self.get_absolute_url()

    def get_absolute_url(self):
        return self.image.url

    def __unicode__(self):
        return u"{entry} - {image}".format(
            entry=truncatechars(self.entry, 10),
            image=truncatechars(self.image.name, 10),
        )



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2015-12-07 23:17
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('sites', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='DemoComment',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('object_pk', models.TextField(verbose_name='object ID')),
                ('user_name', models.CharField(blank=True, max_length=50, verbose_name="user's name")),
                ('user_email', models.EmailField(blank=True, max_length=254, verbose_name="user's email address")),
                ('user_url', models.URLField(blank=True, verbose_name="user's URL")),
                ('comment', models.TextField(max_length=3000, verbose_name='comment')),
                ('submit_date', models.DateTimeField(default=None, verbose_name='date/time submitted')),
                ('ip_address', models.GenericIPAddressField(blank=True, null=True, unpack_ipv4=True, verbose_name='IP address')),
                ('is_public', models.BooleanField(default=True, help_text='Uncheck this box to make the comment effectively disappear from the site.', verbose_name='is public')),
                ('is_removed', models.BooleanField(default=False, help_text='Check this box if the comment is inappropriate. A "This comment has been removed" message will be displayed instead.', verbose_name='is removed')),
                ('content_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='content_type_set_for_democomment', to='contenttypes.ContentType', verbose_name='content type')),
                ('site', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='sites.Site')),
                ('user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='democomment_comments', to=settings.AUTH_USER_MODEL, verbose_name='user')),
            ],
            options={
                'ordering': ('submit_date',),
                'verbose_name': 'comment',
                'verbose_name_plural': 'comments',
                'permissions': [('can_moderate', 'Can moderate comments')],
            },
        ),
    ]



from django.core.urlresolvers import reverse
from django.http import HttpResponseRedirect
from django.views.generic import DetailView

from . import models


class UserProfileDetail(DetailView):
    model = models.UserProfile
    context_object_name = 'profile'
    slug_field = 'user__slug'


def profile_redirect(request):
    if request.user.is_authenticated():
        url = reverse('profile-detail', args=[str(request.user.slug)])
        return HttpResponseRedirect(url)



# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2016-09-17 00:35
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('common', '0004_auto_20151208_0047'),
    ]

    operations = [
        migrations.AlterField(
            model_name='user',
            name='groups',
            field=models.ManyToManyField(blank=True, help_text='The groups this user belongs to. A user will get all permissions granted to each of their groups.', related_name='user_set', related_query_name='user', to='auth.Group', verbose_name='groups'),
        ),
        migrations.AlterField(
            model_name='user',
            name='last_login',
            field=models.DateTimeField(blank=True, null=True, verbose_name='last login'),
        ),
    ]



"""
Django settings for demo project.

For more information on this file, see
https://docs.djangoproject.com/en/1.6/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/1.6/ref/settings/
"""
from os.path import normpath, join

from django import VERSION as DJANGO_VERSION

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
import os
BASE_DIR = os.path.dirname(os.path.dirname(__file__))

# Absolute filesystem path to the top-level project folder:
SITE_ROOT = os.path.dirname(os.path.join(BASE_DIR, 'demo'))

# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/1.6/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = '1g&e2*#444qj833xb2wr0%r0xj2$b_o_d_03nw&zkq%f=1xq(u'

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

""" TEMPLATE CONFIGURATION """

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [
            os.path.normpath(os.path.join(SITE_ROOT, 'templates')),
        ],
        'OPTIONS': {
            'debug': True,
            'context_processors': [
                'django.contrib.auth.context_processors.auth',
                'django.template.context_processors.debug',
                'django.template.context_processors.i18n',
                'django.template.context_processors.media',
                'django.template.context_processors.static',
                'django.template.context_processors.tz',
                'django.contrib.messages.context_processors.messages',
                'django.template.context_processors.request',  # Required for django comment next url, allauth
            ],
            'loaders': [
                'django.template.loaders.filesystem.Loader',
                'django.template.loaders.app_directories.Loader',
                'django.template.loaders.eggs.Loader',
            ]
        },
    },
]

########## END TEMPLATE CONFIGURATION

ALLOWED_HOSTS = []

""" APP CONFIGURATION """
DJANGO_APPS = (
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'django.contrib.sites',  # Required for comments and allauth
    'django.contrib.sitemaps',
)

THIRD_PARTY_APPS = (
    'authtools',  # Custom User classes
    'allauth',  # Login using email
    'allauth.account',
    'allauth.socialaccount',
    'andablog',  # The blog engine
    'django_extensions',  # Misc Tools, we use it for the handy sql reset
    'django_comments',  # Replacement for Django contrib comments
    'bootstrapform',  # Required for bootstrap templates
    'markitup',  # Blog req: For markdown editing
    "taggit",   # Blog req: For tags
)

THIRD_PARTY_APPS += ('debug_toolbar.apps.DebugToolbarConfig',)

# Apps specific for this project go here.
LOCAL_APPS = (
    'common',
    'profiles',
    'democomments',
    'blog',
)

# See: https://docs.djangoproject.com/en/dev/ref/settings/#installed-apps
INSTALLED_APPS = DJANGO_APPS + THIRD_PARTY_APPS + LOCAL_APPS
########## END APP CONFIGURATION

MIDDLEWARE_CLASSES = (
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
)

AUTHENTICATION_BACKENDS = (
    "django.contrib.auth.backends.ModelBackend",  # Needed to login by username in Django admin
    # `allauth` specific authentication methods, such as login by e-mail
    "allauth.account.auth_backends.AuthenticationBackend",
)

ROOT_URLCONF = 'demo.urls'

WSGI_APPLICATION = 'demo.wsgi.application'


# Database
# https://docs.djangoproject.com/en/1.6/ref/settings/#databases

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(BASE_DIR, 'db.sqlite3'),
    }
}

# Internationalization
# https://docs.djangoproject.com/en/1.6/topics/i18n/

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True

""" MEDIA CONFIGURATION """
# See: https://docs.djangoproject.com/en/dev/ref/settings/#media-root
MEDIA_ROOT = normpath(join(SITE_ROOT, 'media'))

# See: https://docs.djangoproject.com/en/dev/ref/settings/#media-url
MEDIA_URL = '/media/'
########## END MEDIA CONFIGURATION

""" STATIC FILE CONFIGURATION """
# See: https://docs.djangoproject.com/en/dev/ref/settings/#static-url
STATIC_URL = '/static/'

# See: https://docs.djangoproject.com/en/dev/ref/contrib/staticfiles/#std:setting-STATICFILES_DIRS
STATICFILES_DIRS = (
    os.path.normpath(os.path.join(SITE_ROOT, 'static')),
)

# See: https://docs.djangoproject.com/en/dev/ref/contrib/staticfiles/#staticfiles-finders
STATICFILES_FINDERS = (
    'django.contrib.staticfiles.finders.FileSystemFinder',
    'django.contrib.staticfiles.finders.AppDirectoriesFinder',
)

DEFAULT_FILE_STORAGE = 'django.core.files.storage.FileSystemStorage'

STATICFILES_STORAGE = 'django.contrib.staticfiles.storage.StaticFilesStorage'
########## END STATIC FILE CONFIGURATION

""" CUSTOM USER CONFIGURATION """
AUTH_USER_MODEL = 'common.User'

""" All Auth """
ACCOUNT_AUTHENTICATION_METHOD = 'email'
ACCOUNT_EMAIL_REQUIRED = True
ACCOUNT_UNIQUE_EMAIL = True
ACCOUNT_USERNAME_REQUIRED = False
ACCOUNT_USER_MODEL_USERNAME_FIELD = None
ACCOUNT_USER_MODEL_EMAIL_FIELD = "email"
ACCOUNT_USER_DISPLAY = lambda user: user.get_short_name()
ACCOUNT_EMAIL_VERIFICATION = "optional"

# Django Auth
LOGIN_REDIRECT_URL = '/profile/'

# See: https://docs.djangoproject.com/en/dev/ref/settings/#site-id
SITE_ID = 1

""" Comments Configuration """
COMMENTS_APP = 'democomments'

""" Markitup! Settings """
MARKITUP_FILTER = ('markdown.markdown', {'safe_mode': False})
MARKITUP_SET = 'markitup/sets/markdown'


# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2015-12-08 00:47
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('common', '0003_auto_20150831_2043'),
    ]

    operations = [
        migrations.AlterField(
            model_name='user',
            name='email',
            field=models.EmailField(max_length=255, unique=True, verbose_name='email address'),
        ),
        migrations.AlterField(
            model_name='user',
            name='is_active',
            field=models.BooleanField(default=True, help_text='Designates whether this user should be treated as active. Unselect this instead of deleting accounts.', verbose_name='active'),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('common', '0002_auto_20150507_1708'),
    ]

    operations = [
        migrations.AlterField(
            model_name='user',
            name='profile_name',
            field=models.CharField(verbose_name='profile name', max_length=20, unique=True),
        ),
    ]



import sys

from django.apps import apps
from django.conf import settings
from django.core.management.base import BaseCommand
from django.db import connections
from django.db.migrations.autodetector import MigrationAutodetector
from django.db.migrations.executor import MigrationExecutor
from django.db.migrations.state import ProjectState
from django.db.utils import OperationalError


class Command(BaseCommand):
    """
    Detect if any apps have missing migration files

    (not necessaily applied though)
    """
    help = "Detect if any apps have missing migration files"

    def handle(self, *args, **kwargs):

        changed = set()
        ignore_list = ['authtools']  # dependencies that we don't care about migrations for (usually for testing only)

        self.stdout.write("Checking...")
        for db in settings.DATABASES.keys():

            try:
                executor = MigrationExecutor(connections[db])
            except OperationalError:
                sys.exit("Unable to check migrations: cannot connect to database\n")

            autodetector = MigrationAutodetector(
                executor.loader.project_state(),
                ProjectState.from_apps(apps),
            )

            changed.update(autodetector.changes(graph=executor.loader.graph).keys())

        for ignore in ignore_list:
            if ignore in changed:
                changed.remove(ignore)

        if changed:
            sys.exit("Apps with model changes but no corresponding migration file: %(changed)s\n" % {
                'changed': list(changed)
            })
        else:
            sys.stdout.write("All migration files present\n")



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from django.test import TestCase
from django.utils import timezone
from django.utils.safestring import SafeText

from andablog import models


class TestEntryModel(TestCase):
    """
    Author Note: We don't do much, if any testing, surrounding the Entry author attribute.
    As all of the integration code for them is in the templates.

    TODO: For 1.7, And-a-Blog should provide a Django system check that ensures that get_short_name and
    get_absolute_url have been implemented on the user model as that is what the templates rely on.
    """

    def setUp(self):
        self.entry = models.Entry.objects.create(title=u'First post!', content=u'The best post on the internet.')

    def test_slug_creation(self):
        """The slug field should automatically get set from the title during post creation"""
        self.assertEqual(self.entry.slug, 'first-post')

    def test__insert_timestamp(self):
        """Ensure the returned value contains a timestamp without going over the max length"""
        # When given the `first-post` slug.
        result = self.entry._insert_timestamp(self.entry.slug)
        self.assertEqual(len(result.split('-')), 3)

        # When given a string > 255 characters.
        slug = '-'.join(['a'] * 250)
        result = self.entry._insert_timestamp(slug)
        self.assertLess(len(result), 255)

    def test_long_slugs_should_not_get_split_midword(self):
        """The slug should not get split mid-word."""
        self.entry.title = SafeText("Please tell me where everyone is getting their assumptions about me?" * 100)
        self.entry.save()
        # The ending should not be a split word.
        self.assertEqual(self.entry.slug[-25:], 'everyone-is-getting-their')

    def test_duplicate_long_slugs_should_get_a_timestamp(self):
        """If a long title has a shortened slug that is a duplicate, it should have a timestamp"""
        self.entry.title = SafeText("Here's a really long title, for testing slug character restrictions")
        self.entry.save()

        duplicate_entry = models.Entry.objects.create(title=self.entry.title, content=self.entry.content)

        self.assertNotEqual(self.entry.slug, duplicate_entry.slug)
        # This is not ideal, but a portion of the original slug is in the duplicate
        self.assertIn(self.entry.slug[:25], duplicate_entry.slug)

    def test_new_duplicate(self):
        """The slug value should automatically be made unique if the slug is taken"""
        duplicate_entry = models.Entry.objects.create(title=self.entry.title, content=self.entry.content)

        self.assertNotEqual(self.entry.slug, duplicate_entry.slug)
        self.assertIn(self.entry.slug, duplicate_entry.slug)

    def test_title_rename_to_duplicate(self):
        """Upon title rename, the slug value should automatically be made unique if the slug is taken"""
        new_entry_2 = models.Entry.objects.create(title=u'Second post!', content=u'Second best post on the internet.')

        # Rename
        new_entry_2.title = self.entry.title
        new_entry_2.save()

        self.assertNotEqual(self.entry.slug, new_entry_2.slug)
        self.assertIn(self.entry.slug, new_entry_2.slug)

    def test_publishing(self):
        """Test publishing an entry"""
        self.assertFalse(self.entry.is_published)
        self.assertFalse(self.entry.published_timestamp)

        moment_ago = timezone.now()

        self.entry.is_published = True
        self.entry.save()

        self.assertGreaterEqual(self.entry.published_timestamp, moment_ago)

    def test_unpublishing(self):
        """Should be able to pull an entry after it has been published"""
        self.entry.is_published = True
        self.entry.save()

        self.entry.is_published = False
        self.entry.save()

        self.assertFalse(self.entry.is_published)
        self.assertIsNone(self.entry.published_timestamp)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import six
from django.core.urlresolvers import reverse
from django.template.defaultfilters import truncatewords
from django.test import TestCase

from andablog import models, feeds


class TestLatestEntriesFeed(TestCase):

    def setUp(self):
        self.post_1 = models.Entry.objects.create(title=u'Welcome!', is_published=True)
        self.post_2 = models.Entry.objects.create(title=u'Busy Busy', is_published=True)
        self.post_3 = models.Entry.objects.create(title=u'Last Post', is_published=True)
        self.post_4 = models.Entry.objects.create(title=u'Back again!')

        self.feed = feeds.LatestEntriesFeed()

    def test_feed_properties(self):
        """The base entry feed should return the last 10 entries by default"""
        actual_entries = self.feed.items()

        expected_slugs = ['last-post', 'busy-busy', 'welcome']
        actual_slugs = [entry.slug for entry in actual_entries]

        self.assertEqual(self.feed.link(), reverse('andablog:entrylist'))
        self.assertEqual(actual_slugs, expected_slugs)
        self.assertNumQueries(1)

    def test_feed_max(self):
        """Should only return ten by default"""
        for x in range(8):
            models.Entry.objects.create(title=u'Ni' + six.text_type(x), is_published=True)

        self.assertEqual(self.feed.items().count(), 10)


class TestLatestEntriesFeedItem(TestCase):

    def setUp(self):
        self.entry = models.Entry.objects.create(title=u'Welcome!', is_published=True)

        self.feed = feeds.LatestEntriesFeed()

    def test_item_properites(self):
        """A base entry feed should implement all item specific properties"""
        expected = (
            self.entry.published_timestamp,
            self.entry.title,
            None,
            None,
            None,
            truncatewords(self.entry.content, 26)
        )
        actual = (
            self.feed.item_pubdate(self.entry),
            self.feed.item_title(self.entry),
            self.feed.item_author_name(self.entry),
            self.feed.item_author_email(self.entry),
            self.feed.item_author_link(self.entry),
            self.feed.item_description(self.entry),
        )

        #NOTE: Populated author properties testing is covered in the demo site tests
        self.assertEqual(expected, actual)



from django.db.models import Q
from django.views.generic import ListView, DetailView

from . import models


class EntriesList(ListView):

    model = models.Entry
    template_name = 'andablog/entry_list.html'
    context_object_name = 'entries'
    paginate_by = 10
    paginate_orphans = 5

    def get_queryset(self):
        queryset = super(EntriesList, self).get_queryset().filter(
            Q(is_published=True) | Q(author__isnull=False, author=self.request.user.id))
        return queryset.order_by('is_published', '-published_timestamp')  # Put 'drafts' first.


class EntryDetail(DetailView):

    model = models.Entry
    template_name = 'andablog/entry_detail.html'
    context_object_name = 'entry'
    slug_field = 'slug'

    def get_queryset(self):
        return super(EntryDetail, self).get_queryset().filter(
            Q(is_published=True) | Q(author__isnull=False, author=self.request.user.id))



"""
NOTE: Can not import models at module level because all app model loading must first be finished.
"""

def get_model():
    from . import models
    return models.DemoComment


def get_form():
    from . import forms
    return forms.DemoCommentForm



from django.conf.urls import url

from . import views

app_name = 'andablog'
urlpatterns = [
    url('^$', views.EntriesList.as_view(), name='entrylist'),
    url(r'^(?P<slug>[A-Za-z0-9-_]+)/$', views.EntryDetail.as_view(), name='entrydetail'),
]


from django.test import TestCase

from . import models
from . import sitemaps
from django.utils import timezone


class TestProfilePage(TestCase):

    fixtures = ['one_user', 'one_profile']

    def setUp(self):
        self.profile = models.UserProfile.objects.get(user__slug=u'superman')
        self.url = self.profile.get_absolute_url()

    def test_anonymous_get(self):
        """Should be able to view a user's profile page"""
        response = self.client.get(self.url)

        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.context['profile'], self.profile)
        self.assertNumQueries(1)


class TestUserProfileSitemap(TestCase):

    fixtures = ['three_users', 'three_profiles']

    def setUp(self):
        self.userprofile_map = sitemaps.UserProfileSitemap()
        self.wonder_woman = models.UserProfile.objects.get(user__slug='wonder-woman')

    def test_items(self):
        """All active user profiles should be listed, sorted by most recently joined"""
        # It's a trap!
        self.wonder_woman.user.is_active = False
        self.wonder_woman.user.save()

        actual_profiles = self.userprofile_map.items()

        expected_slugs = ['agent-0014', 'superman']
        actual_slugs = [profile.user.slug for profile in actual_profiles]

        self.assertEqual(actual_slugs, expected_slugs)
        self.assertNumQueries(1)

    def test_last_modified(self):
        """Should be able to get the last update from a profile"""
        superman = models.UserProfile.objects.get(user__slug='superman')

        actual_update = self.userprofile_map.lastmod(self.wonder_woman)

        now = timezone.now()
        self.assertGreaterEqual(now, actual_update)
        self.assertGreaterEqual(actual_update, superman.modified)



import six
from django.core.urlresolvers import reverse
from django.db import models
from django.conf import settings
from model_utils.models import TimeStampedModel


class UserProfile(TimeStampedModel):

    user = models.OneToOneField(settings.AUTH_USER_MODEL)
    greatest_fear = models.CharField(
        blank=True,
        null=True,
        max_length=100,
    )
    home = models.TextField(
        blank=True,
        null=True,
    )

    def get_short_name(self):
        return self.user

    def get_absolute_url(self):
        return reverse('profile-detail', args=[str(self.user.slug)])

    def __unicode__(self):
        return six.text_type(self.get_short_name())



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('andablog', '0004_shorten_entry_title'),
    ]

    operations = [
        migrations.AlterField(
            model_name='entry',
            name='slug',
            field=models.SlugField(unique=True, editable=False, max_length=255),
        ),
        migrations.AlterField(
            model_name='entry',
            name='title',
            field=models.CharField(max_length=255),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import time
from django.db import migrations, models


# The new, maximum length of titles.
TITLE_LENGTH = 255


def truncate_entry_titles(apps, schema_editor):
    """This function will truncate the values of Entry.title so they are
    255 characters or less.
    """
    Entry = apps.get_model("andablog", "Entry")

    for entry in Entry.objects.all():
        # Truncate to 255 characters (or less) but keep whole words intact.
        while len(entry.title) > TITLE_LENGTH:
            entry.title = ' '.join(entry.title.split()[:-1])
        entry.save()


class Migration(migrations.Migration):

    dependencies = [
        ('andablog', '0003_auto_20150826_2353'),
    ]

    operations = [
        migrations.RunPython(truncate_entry_titles)
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('andablog', '0002_auto_20141204_1659'),
    ]

    operations = [
        migrations.AlterField(
            model_name='entryimage',
            name='image',
            field=models.ImageField(blank=True, upload_to='andablog/images'),
        ),
    ]



from django.contrib import admin
from django.db.models import Q
from django.contrib.auth import get_user_model

from .models import Entry, EntryImage


class EntryImageInline(admin.TabularInline):
    model = EntryImage
    extra = 1
    readonly_fields = ['image_url']


class EntryAdmin(admin.ModelAdmin):

    list_display = (
        'title',
        'is_published',
        'published_timestamp',
    )
    search_fields = (
        'title',
        'content',
    )
    readonly_fields = (
        'slug',
        'published_timestamp',
    )

    inlines = [
        EntryImageInline,
    ]

    def formfield_for_foreignkey(self, db_field, request, **kwargs):
        if db_field.name == "author":
            kwargs["queryset"] = get_user_model().objects.filter(
                Q(is_superuser=True) | Q(user_permissions__content_type__app_label='andablog',
                                         user_permissions__content_type__model='entry')).distinct()
            kwargs['initial'] = request.user.id
        return super(EntryAdmin, self).formfield_for_foreignkey(db_field, request, **kwargs)


class EntryImageAdmin(admin.ModelAdmin):
    pass

admin.site.register(Entry, EntryAdmin)
admin.site.register(EntryImage, EntryImageAdmin)



from django.contrib.syndication.views import Feed
from django.core.urlresolvers import reverse
from django.template.defaultfilters import truncatewords_html

from .models import Entry


class LatestEntriesFeed(Feed):
    """
    This is just a base class for a blog entry field.
    It should be sub-classed and further refined before use.
    """
    MAX_ENTRIES = 10

    title = 'Latest blog entries'
    description = 'Latest blog entries sorted by newest to oldest.'

    def item_pubdate(self, item):
        return item.published_timestamp

    def item_author_name(self, item):
        if item.author is None:
            return None
        return item.author.get_short_name()

    def item_author_email(self, item):
        if item.author is None:
            return None
        return item.author.email

    def item_author_link(self, item):
        if item.author is None:
            return None
        return item.author.get_absolute_url()

    def link(self):
        return reverse('andablog:entrylist')

    def item_description(self, item):
        # TODO: "Better support for truncating markup" #2
        return truncatewords_html(item.content, 26)

    def item_title(self, item):
        return item.title

    def items(self):
        return Entry.objects.filter(is_published=True).order_by('-published_timestamp')[:self.MAX_ENTRIES]



from django.conf import settings
from django.db import models
from django.utils import timezone
from django.utils.translation import ugettext_lazy as _

from django_comments.managers import CommentManager
from django_comments.models import BaseCommentAbstractModel

COMMENT_MAX_LENGTH = getattr(settings, 'COMMENT_MAX_LENGTH', 3000)


class RelatedCommentManager(CommentManager):

    def get_queryset(self):
        return super(RelatedCommentManager, self).get_queryset().select_related(
            'user',
            'user__userprofile'
        )


# Ended up not using the concrete model because of this new limitation: https://code.djangoproject.com/ticket/22872
# Not a big deal as this is mainly meant for a demo of a custom comment solution anyway.
class DemoComment(BaseCommentAbstractModel):

    # Who posted this comment? If ``user`` is set then it was an authenticated
    # user; otherwise at least user_name should have been set and the comment
    # was posted by a non-authenticated user.
    user = models.ForeignKey(settings.AUTH_USER_MODEL, verbose_name=_('user'),
                             blank=True, null=True, related_name="%(class)s_comments")
    user_name = models.CharField(_("user's name"), max_length=50, blank=True)
    # Explicit `max_length` to apply both to Django 1.7 and 1.8+.
    user_email = models.EmailField(_("user's email address"), max_length=254,
                                   blank=True)
    user_url = models.URLField(_("user's URL"), blank=True)

    comment = models.TextField(_('comment'), max_length=COMMENT_MAX_LENGTH)

    # Metadata about the comment
    submit_date = models.DateTimeField(_('date/time submitted'), default=None)
    ip_address = models.GenericIPAddressField(_('IP address'), unpack_ipv4=True, blank=True, null=True)
    is_public = models.BooleanField(_('is public'), default=True,
                                    help_text=_('Uncheck this box to make the comment effectively '
                                                'disappear from the site.'))
    is_removed = models.BooleanField(_('is removed'), default=False,
                                     help_text=_('Check this box if the comment is inappropriate. '
                                                 'A "This comment has been removed" message will '
                                                 'be displayed instead.'))

    class Meta:
        ordering = ('submit_date',)
        permissions = [("can_moderate", "Can moderate comments")]
        verbose_name = _('comment')
        verbose_name_plural = _('comments')

    objects = RelatedCommentManager()

    def __str__(self):
        return "%s: %s..." % (self.name, self.comment[:50])

    def save(self, *args, **kwargs):
        if self.submit_date is None:
            self.submit_date = timezone.now()
        super(DemoComment, self).save(*args, **kwargs)

    def _get_userinfo(self):
        """
        Get a dictionary that pulls together information about the poster
        safely for both authenticated and non-authenticated comments.

        This dict will have ``name``, ``email``, and ``url`` fields.
        """
        if not hasattr(self, "_userinfo"):
            userinfo = {
                "name": self.user_name,
                "email": self.user_email,
                "url": self.user_url
            }
            if self.user_id:
                u = self.user
                if u.email:
                    userinfo["email"] = u.email

                # If the user has a full name, use that for the user name.
                # However, a given user_name overrides the raw user.username,
                # so only use that if this comment has no associated name.
                if u.get_full_name():
                    userinfo["name"] = self.user.get_full_name()
                elif not self.user_name:
                    userinfo["name"] = u.get_username()
            self._userinfo = userinfo
        return self._userinfo

    userinfo = property(_get_userinfo, doc=_get_userinfo.__doc__)

    def _get_name(self):
        return self.userinfo["name"]

    def _set_name(self, val):
        if self.user_id:
            raise AttributeError(_("This comment was posted by an authenticated "
                                   "user and thus the name is read-only."))
        self.user_name = val

    name = property(_get_name, _set_name, doc="The name of the user who posted this comment")

    def _get_email(self):
        return self.userinfo["email"]

    def _set_email(self, val):
        if self.user_id:
            raise AttributeError(_("This comment was posted by an authenticated "
                                   "user and thus the email is read-only."))
        self.user_email = val

    email = property(_get_email, _set_email, doc="The email of the user who posted this comment")

    def _get_url(self):
        return self.userinfo["url"]

    def _set_url(self, val):
        self.user_url = val

    url = property(_get_url, _set_url, doc="The URL given by the user who posted this comment")

    def get_absolute_url(self, anchor_pattern="#c%(id)s"):
        return self.get_content_object_url() + (anchor_pattern % self.__dict__)

    def get_as_text(self):
        """
        Return this comment as plain text.  Useful for emails.
        """
        d = {
            'user': self.user or self.name,
            'date': self.submit_date,
            'comment': self.comment,
            'domain': self.site.domain,
            'url': self.get_absolute_url()
        }
        return _('Posted by %(user)s at %(date)s\n\n%(comment)s\n\nhttp://%(domain)s%(url)s') % d